@@ -54,7 +54,12 @@ class ProgressLogger:
                 )
 
     def log_progress_event(
-        self, event_name: str, current: int, total: int, event_format_string: str
+        self,
+        event_name: str,
+        current: int,
+        total: int,
+        event_format_string: str,
+        backoff: bool = True,
     ) -> None:
         self.logger.info(event_format_string.format(event_name, current, total))
         if self.event_loggers:
@@ -64,6 +69,7 @@ class ProgressLogger:
                     current=current,
                     total=total,
                     event_format_string=event_format_string,
+                    backoff=backoff,
                 )
 
     def log_event(self, event_name: str, event_text: str) -> None:
